Type DCB
	Id As String * 1        '  identifies the comm port
	BaudRate As Integer     '  data transfer rate
	ByteSize As String * 1  '  bits per data character
	Parity As String * 1    '  constant specifies type of parity
	StopBits As String * 1  '  constant specifies number of stop bits
	RlsTimeout As Integer   '  milliseconds to wait for CD
	CtsTimeout As Integer   '  milliseconds to wait for CTS input
	DsrTimeout As Integer   '  milliseconds to wait for DSR input
	Bits1 As String * 1     '  sum of DCB1 bits
	Bits2 As String * 1     '  sum of DCB2 bits
	XonChar As String * 1   '  XON character usually ^Q ascii17
	XoffChar As String * 1  '  XOFF character usually ^S ascii19
	XonLim As Integer       '  send XON/DTR=1/RTS=1 when chars in rx buffer fall below this value
	XoffLim As Integer      '  send XOFF/DTR=0/RTS=0 when chars in rx buffer exceed this value
	PeChar As String * 1    '  character to sub for errored characters (ignore if fParity=0 or fPeChar=0)
	EofChar As String * 1   '  EOF character usually ^Z CHR$(26) (ignored if fBinary=1)
	EvtChar As String * 1   '  immediate event character usually ^C ascii3 (ignore if fChEvt=0)
	TxDelay As Integer      '  not implemented
End Type

Type COMSTAT
	Bits As String * 1      '  see CSTF constants
	cbInQue As Integer      '  number of characters in input buffer
	cbOutQue As Integer     '  number of characters in output buffer
End Type


''  COMM declarations
'
Global Const NOPARITY = 0
Global Const ODDPARITY = 1
Global Const EVENPARITY = 2
Global Const MARKPARITY = 3
Global Const SPACEPARITY = 4

Global Const ONESTOPBIT = 0
Global Const ONE5STOPBITS = 1
Global Const TWOSTOPBITS = 2

Global Const IGNORE = 0         '  Ignore signal
Global Const INFINITE = &HFFFF  '  Infinite timeout

''  COMM Error Flags
Global Const CE_RXOVER = &H1    '  Receive Queue overflow
Global Const CE_OVERRUN = &H2   '  Receive Overrun Error
Global Const CE_RXPARITY = &H4  '  Receive Parity Error
Global Const CE_FRAME = &H8     '  Receive Framing error
Global Const CE_BREAK = &H10    '  Break Detected
Global Const CE_CTSTO = &H20    '  CTS Timeout
Global Const CE_DSRTO = &H40    '  DSR Timeout
Global Const CE_RLSDTO = &H80   '  RLSD Timeout
Global Const CE_TXFULL = &H100  '  TX Queue is full
Global Const CE_PTO = &H200     '  LPTx Timeout
Global Const CE_IOE = &H400     '  LPTx I/O Error
Global Const CE_DNS = &H800     '  LPTx Device not selected
Global Const CE_OOP = &H1000    '  LPTx Out-Of-Paper
Global Const CE_MODE = &H8000   '  Requested mode unsupported

Global Const IE_BADID = (-1)        '  Invalid or unsupported id
Global Const IE_OPEN = (-2)         '  Device Already Open
Global Const IE_NOPEN = (-3)        '  Device Not Open
Global Const IE_MEMORY = (-4)       '  Unable to allocate queues
Global Const IE_DEFAULT = (-5)      '  Error in default parameters
Global Const IE_HARDWARE = (-10)    '  Hardware Not Present
Global Const IE_BYTESIZE = (-11)    '  Illegal Byte Size
Global Const IE_BAUDRATE = (-12)    '  Unsupported BaudRate

''  COMM Events
Global Const EV_RXCHAR = &H1    '  Any Character received
Global Const EV_RXFLAG = &H2    '  Received certain character
Global Const EV_TXEMPTY = &H4   '  Transmit Queue Empty
Global Const EV_CTS = &H8       '  CTS changed state
Global Const EV_DSR = &H10      '  DSR changed state
Global Const EV_RLSD = &H20     '  RLSD changed state
Global Const EV_BREAK = &H40    '  BREAK received
Global Const EV_ERR = &H80      '  Line status error occurred
Global Const EV_RING = &H100    '  Ring signal detected
Global Const EV_PERR = &H200    '  Printer error occured
Global Const EV_CTSS = &H400    '  Current state of CTS
Global Const EV_DSRS = &H800    '  Current state of DSR
Global Const EV_RLSDS = &H1000  '  Current state of RLSD signal


''  COMM Escape Functions
Global Const SETXOFF = 1       '  Simulate XOFF received
Global Const SETXON = 2        '  Simulate XON received
Global Const SETRTS = 3        '  Set RTS high
Global Const CLRRTS = 4        '  Set RTS low
Global Const SETDTR = 5        '  Set DTR high
Global Const CLRDTR = 6        '  Set DTR low
Global Const RESETDEV = 7      '  Reset device if possible (only LPT)
Global Const GETMAXLPT = 8
Global Const GETMAXCOM = 9
Global Const GETBASEIRQ = 10

''  COMM Speed Optional Constants
Global Const CBR_110 = &HFF10  'uint= 65296 int= -240 int= 110
Global Const CBR_300 = &HFF11  'uint= 65297 int= -239 int= 300
Global Const CBR_600 = &HFF12  'uint= 65298 int= -238 int= 600
Global Const CBR_1200 = &HFF13  'uint= 65299 int= -237 int= 1200
Global Const CBR_2400 = &HFF14  'uint= 65300 int= -236 int= 2400
Global Const CBR_4800 = &HFF15  'uint= 65301 int= -235 int= 4800
Global Const CBR_9600 = &HFF16  'uint= 65302 int= -234 int= 9600
Global Const CBR_14400 = &HFF17  'uint= 65303 int= -233 int= 14400
Global Const CBR_19200 = &HFF18  'uint= 65304 int= -232 int= 19200
Global Const CBR_38400 = &HFF1B  'uint= 65307 int= -229 uint=38400 int= -27136
Global Const CBR_56000 = &HFF1F  'uint= 65311 int= -225 uint= 57600 int= -7936
'Speed 115,200 is any value uint from 57600 to 65929 or int from -7936 to -240


'' COMM notifications on WM_COMMNOTIFY messages
Global Const CN_RECEIVE = &H1
Global Const CN_TRANSMIT = &H2
Global Const CN_EVENT = &H4

'' COMM status flags
Global Const CSTF_CTSHOLD = &H1     '  tx on hold waiting for CTS
Global Const CSTF_DSRHOLD = &H2     '  tx on hold waiting for DSR
Global Const CSTF_RLSDHOLD = &H4    '  tx on hold waiting for CD (RLSD)
Global Const CSTF_XOFFHOLD = &H8    '  tx on hold waiting for XON
Global Const CSTF_XOFFSENT = &H10   '  waiting because XOFF sent, afraid more tx will be interpreted as XON
Global Const CSTF_EOF = &H20        '  received EOF character
Global Const CSTF_TXIM = &H40       '  character waiting to be transmitted

'' COMM DCB Bits1
Global Const DCB1_BINARY = &H1          '  Binary mode ignores EOF character
Global Const DCB1_RTSDISABLE = &H2      '  RTS disable (Don't send RTS when port opened)
Global Const DCB1_PARITY = &H4          '  Turn on parity checking
Global Const DCB1_OUTXCTSFLOW = &H8     '  Use CTS for flow control (Stop tx if CTS not present)
Global Const DCB1_OUTXDSRFLOW = &H10    '  Use DSR for flow control (Stop tx if DSR not present)
Global Const DCB1_DTRDISABLE = &H80     '  DTR disable (Don't send DTR when port opened)

'' COMM DCB Bits2
Global Const DCB2_OUTX = &H1        '  Use XON/XOFF for flow control (Stop/start tx on XOFF/XON reception)
Global Const DCB2_INX = &H2         '  Other guy monitors XON/XOFF. Send them when rx buffer fills/empties
Global Const DCB2_PECHAR = &H4      '  If DCB1_PARITY, substitute DCB.PeChar  for errored characters
Global Const DCB2_NULL = &H8        '  Discard null characters (ascii 0)
Global Const DCB2_CHEVT = &H10      '  Trigger immediate event if receive DCB.EvtChar
Global Const DCB2_DTRFLOW = &H20    '  Use DTR for flow control. Toggle when rx buffer fills/empties
Global Const DCB2_RTSFLOW = &H40    '  Use RTS for flow control. Toggle when rx buffer fills/empties

Global Const LPTx = &H80    '  Set if ID is for LPT device

Global Const WM_COMMNOTIFY = &H44

Declare Function OpenComm Lib "User" (ByVal lpComName As String, ByVal wInQueue As Integer, ByVal wOutQueue As Integer) As Integer
Declare Function SetCommState Lib "User" (lpDCB As DCB) As Integer
Declare Function ReadComm Lib "User" (ByVal nCid As Integer, ByVal lpBuf As String, ByVal nSize As Integer) As Integer
Declare Function WriteComm Lib "User" (ByVal nCid As Integer, ByVal lpBuf As String, ByVal nSize As Integer) As Integer
Declare Function CloseComm Lib "User" (ByVal nCid As Integer) As Integer
Declare Function BuildCommDCB Lib "User" (ByVal lpDef As String, lpDCB As DCB) As Integer
Declare Function GetCommError Lib "User" (ByVal nCid As Integer, Lpstat As COMSTAT) As Integer
Declare Function EnableCommNotification Lib "User" (ByVal idComDev As Integer, ByVal hWnd As Integer, ByVal cbWriteNotify As Integer, ByVal OutQueue As Integer) As Integer
Declare Function GetCommState Lib "User" (ByVal nCid%, lpDCB As DCB) As Integer
Declare Function SetCommEventMask Lib "User" (ByVal nCid As Integer, ByVal nEvtMask As Integer) As Long
Declare Function EscapeCommFunction Lib "User" (ByVal nCid As Integer, ByVal nFunc As Integer) As Integer
''   Functions for Scrolling a Window
'
Declare Sub ScrollWindow Lib "User" (ByVal hWnd As Integer, ByVal XAmount As Integer, ByVal Yamount As Integer, lpRect As Any, lpClipRect As Any)
Declare Sub UpdateWindow Lib "User" (ByVal hWnd As Integer)

Function DisplayBits1 (Bits1 As String) As String
'Displays the "Bits1" parameters of a DCB

Dim CRLF As String
Dim Bits As Integer

  CRLF = Chr$(13) & Chr$(10)
  Bits = Asc(Bits1)

    TempShow$ = ""
    If Bits And 1 Then
      TempShow$ = TempShow$ & "Binary Mode: EOF ignored. " & CRLF
    Else
      TempShow$ = TempShow$ & "Text Mode: Respond to EOF. " & CRLF
    End If

    If Bits And 2 Then
      TempShow$ = TempShow$ & "RTS is disabled. " & CRLF
    Else
      TempShow$ = TempShow$ & "RTS is enabled. " & CRLF
    End If
    
    If Bits And 4 Then
      TempShow$ = TempShow$ & "Parity checking is on. " & CRLF
    Else
      TempShow$ = TempShow$ & "Parity checking is off. " & CRLF
    End If
    
    If Bits And 8 Then TempShow$ = TempShow$ & "Flow: Stop TX if no CTS. " & CRLF
    
    If Bits And 16 Then TempShow$ = TempShow$ & "Flow: Stop TX if no DSR. " & CRLF
    
    If Bits And 128 Then
      TempShow$ = TempShow$ & "DTR is disabled. "
    Else
      TempShow$ = TempShow$ & "DTR is enabled. "
    End If

    DisplayBits1 = TempShow$

End Function

Function DisplayBits2 (Bits2 As String) As String
'Displays the "Bits2" parameters of a DCB
  
Dim Bits As Integer
Dim CRLF As String

  CRLF = Chr$(13) & Chr$(10)
  Bits = Asc(Bits2)

    TempShow$ = ""
    If Bits And 1 Then TempShow$ = TempShow$ & "Flow: Stop TX if XOFF rcvd. " & CRLF

    If Bits And 2 Then TempShow$ = TempShow$ & "Flow: Send XOFF when RX buffer fills. " & CRLF
    
    If Bits And 4 Then
      TempShow$ = TempShow$ & "If Parity on, substitute bad characters. " & CRLF
    Else
      TempShow$ = TempShow$ & "Do not substitute bad parity characters. " & CRLF
    End If
    
    If Bits And 8 Then
      TempShow$ = TempShow$ & "Discard null characters. " & CRLF
    Else
      TempShow$ = TempShow$ & "Do not discard null characters. " & CRLF
    End If
    
    If Bits And 16 Then
      TempShow$ = TempShow$ & "Trigger event if receiving event character. " & CRLF
    Else
      TempShow$ = TempShow$ & "Ignore reception of event character. " & CRLF
    End If

    If Bits And 32 Then TempShow$ = TempShow$ & "Flow: Toggle DTR when RX buffer fills. " & CRLF

    If Bits And 64 Then TempShow$ = TempShow$ & "Flow: Toggle RTS when RX buffer fills. " & CRLF


    DisplayBits2 = Left$(TempShow$, Len(TempShow$) - 2)  'Strip final CRLF off


End Function

Function DisplayCommError (CommError As Integer) As String
'Displays the errors detected by the API function "GetCommError"

Dim CRLF As String

    CRLF = Chr$(13) & Chr$(10)

    If CommError = 0 Then
      DisplayCommError = "No Reported Comm Errors. "
    ElseIf ((CommError < 0) Or (CommError > 40959)) Then
      DisplayCommError = "Unanticipated GetCommError #" & Str$(CommError)
    Else
      TempError$ = ""
      If CommError And 1 Then TempError$ = TempError$ + "Receive Queue overflow. " & CRLF
      If CommError And 2 Then TempError$ = TempError$ + "Receive Overrun Error. " & CRLF
      If CommError And 4 Then TempError$ = TempError$ + "Receive Parity Error. " & CRLF
      If CommError And 8 Then TempError$ = TempError$ + "Receive Framing error. " & CRLF
      If CommError And 16 Then TempError$ = TempError$ + "Break Detected. " & CRLF
      If CommError And 32 Then TempError$ = TempError$ + "CTS Timeout. " & CRLF
      If CommError And 64 Then TempError$ = TempError$ + "DSR Timeout. " & CRLF
      If CommError And 128 Then TempError$ = TempError$ + "RLSD Timeout. " & CRLF
      If CommError And 256 Then TempError$ = TempError$ + "TX Queue is full. " & CRLF
      If CommError And 512 Then TempError$ = TempError$ + "LPTx Timeout. " & CRLF
      If CommError And 1024 Then TempError$ = TempError$ + "LPTx I/O Error. " & CRLF
      If CommError And 2048 Then TempError$ = TempError$ + "LPTx Device not selected. " & CRLF
      If CommError And 4096 Then TempError$ = TempError$ + "LPTx Out-Of-Paper. " & CRLF
      If CommError And 32768 Then TempError$ = TempError$ + "Requested mode unsupported. " & CRLF
      DisplayCommError = Left$(TempError$, Len(TempError$) - 2)  'Strip the trailing CRLF off
    End If

End Function

Function DisplayCommEvent (CommEvent As Integer) As String
'Displays the events detected by the API function "GetCommMask"

Dim CRLF As String

    CRLF = Chr$(13) & Chr$(10)

    If CommEvent = 0 Then
      DisplayCommEvent = "No masked events detected"
      Exit Function
    ElseIf ((CommEvent < 0) Or (CommEvent > 8191)) Then
      DisplayCommEvent = "Unanticipated GetCommMask event #" & Str$(CommEvent)
      Exit Function
    End If
    
    TempMask$ = ""
    If CommEvent And 1 Then TempMask$ = TempMask$ & "Character received. " & CRLF
    If CommEvent And 2 Then TempMask$ = TempMask$ & "Received event character. " & CRLF
    If CommEvent And 4 Then TempMask$ = TempMask$ & "Transmit Queue Empty. " & CRLF
    If CommEvent And 8 Then TempMask$ = TempMask$ & "CTS changed state. " & CRLF
    If CommEvent And 16 Then TempMask$ = TempMask$ & "DSR changed state. " & CRLF
    If CommEvent And 32 Then TempMask$ = TempMask$ & "RLSD (CD) changed state. " & CRLF
    If CommEvent And 64 Then TempMask$ = TempMask$ & "BREAK received. " & CRLF
    If CommEvent And 128 Then TempMask$ = TempMask$ & "Serial device error occurred. " & CRLF
    If CommEvent And 256 Then TempMask$ = TempMask$ & "Ring signal detected. " & CRLF
    If CommEvent And 512 Then TempMask$ = TempMask$ & "Parallel device error occured. " & CRLF
    
    If CommEvent And 1024 Then
      TempMask$ = TempMask$ & "CTS is high (inactive). " & CRLF
    Else
      TempMask$ = TempMask$ & "CTS is low (active). " & CRLF
    End If
    
    If CommEvent And 2048 Then
      TempMask$ = TempMask$ + "DSR is high (inactive). " & CRLF
    Else
      TempMask$ = TempMask$ + "DSR is low (active). " & CRLF
    End If
    
    If CommEvent And 4096 Then
      TempMask$ = TempMask$ + "RLSD (CD) signal is high (inactive). "
    Else
      TempMask$ = TempMask$ + "RLSD (CD) signal is low (active). "
    End If

    DisplayCommEvent = TempMask$
  

End Function

Function DisplayCOMSTAT (PassedCOMSTAT As COMSTAT) As String
'Displays the status detected by the API function "GetCommError"

Dim Bits As Integer
Dim CRLF As String

    CRLF = Chr$(13) & Chr$(10)
    Bits = Asc(PassedCOMSTAT.Bits)

    If Bits < 0 Or Bits > 127 Then
       DisplayCOMSTAT = "Unexpected COMSTAT value" & Str$(Bits)
    ElseIf Bits = 0 Then
       DisplayCOMSTAT = "TX is not on hold. "
    Else
      TempShow$ = ""
      If Bits And 1 Then TempShow$ = TempShow$ & "TX on hold waiting for CTS. " & CRLF
      If Bits And 2 Then TempShow$ = TempShow$ & "TX on hold waiting for DSR. " & CRLF
      If Bits And 4 Then TempShow$ = TempShow$ & "TX on hold waiting for CD / RLSD. " & CRLF
      If Bits And 8 Then TempShow$ = TempShow$ & "TX on hold waiting for XON. " & CRLF
      If Bits And 16 Then TempShow$ = TempShow$ & "XOFF sent. TX on hold as precaution. " & CRLF
      If Bits And 32 Then TempShow$ = TempShow$ & "Received EOF character. " & CRLF
      If Bits And 64 Then TempShow$ = TempShow$ & "Character waiting to be transmitted. " & CRLF
      TempShow$ = TempShow$ & "Input Queue level =" & Str$(PassedCOMSTAT.cbInQue) & CRLF
      TempShow$ = TempShow$ & "Output Queue level =" & Str$(PassedCOMSTAT.cbOutQue)
      DisplayCOMSTAT = TempShow$
    End If

End Function

Function DisplayDCB (PassedDCB As DCB) As String
'Used to display data in the "DCB" from API functions "BuildCommDCB" and "GetCommState"

Dim CRLF As String

  CRLF = Chr$(13) & Chr$(10)

  TempDCB$ = "nCid =" & Asc(PassedDCB.Id)
  Select Case PassedDCB.BaudRate
    Case Is > -1
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate =" & Str$(Int(115200 / (Int(115200 / PassedDCB.BaudRate))))
    Case Is = CBR_110
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate = 110"
    Case Is = CBR_300
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate = 300"
    Case Is = CBR_600
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate = 600"
    Case Is = CBR_1200
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate = 1200"
    Case Is = CBR_2400
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate = 2400"
    Case Is = CBR_4800
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate = 4800"
    Case Is = CBR_9600
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate = 9600"
    Case Is = CBR_14400
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate = 14400"
    Case Is = CBR_19200
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate = 19200"
    Case Is = CBR_38400
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate = 38400"
    Case Is = CBR_56000
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate = 57600"
    Case Is < -7935
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate =" & Str$(Int(115200 / (Int(115200 / (PassedDCB.BaudRate + 65536)))))
    Case Is < -240
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate = 115200"
    Case Else
      TempDCB$ = TempDCB$ & CRLF & "Baud Rate =" & Str$(Int(115200 / (Int(115200 / PassedDCB.BaudRate))))
  End Select
  TempDCB$ = TempDCB$ & CRLF & "Data Bytes =" & Str$(Asc(PassedDCB.ByteSize))
  TempDCB$ = TempDCB$ & CRLF & "Parity = " & Mid$("NONE ODD  EVEN MARK SPACE", (Val(PassedDCB.Parity) + 1), 5)
  TempDCB$ = TempDCB$ & CRLF & "Stop Bits = " & Mid$("ONE1.5TWO", (Val(PassedDCB.StopBits) + 1), 3)
  TempDCB$ = TempDCB$ & CRLF & "RLS/CD recognize time (mSec) =" & Str$(PassedDCB.RlsTimeout)
  TempDCB$ = TempDCB$ & CRLF & "CTS recognize time (mSec) =" & Str$(PassedDCB.CtsTimeout)
  TempDCB$ = TempDCB$ & CRLF & "DSR recognize time (mSec) =" & Str$(PassedDCB.DsrTimeout)
  TempDCB$ = TempDCB$ & CRLF & "Bits1 decimal =" & Str$(Asc(PassedDCB.Bits1))
  TempDCB$ = TempDCB$ & CRLF & "Bits2 decimal =" & Str$(Asc(PassedDCB.Bits2))
  TempDCB$ = TempDCB$ & CRLF & "XON character decimal =" & Str$(Asc(PassedDCB.XonChar))
  TempDCB$ = TempDCB$ & CRLF & "XOFF character decimal =" & Str$(Asc(PassedDCB.XoffChar))
  TempDCB$ = TempDCB$ & CRLF & "Send Flow ON when rx buffer falls to =" & Str$(PassedDCB.XonLim)
  TempDCB$ = TempDCB$ & CRLF & "Send Flow OFF when rx buffer fills to =" & Str$(PassedDCB.XoffLim)
  TempDCB$ = TempDCB$ & CRLF & "Errored Parity Substitution character decimal =" & Str$(Asc(PassedDCB.PeChar))
  TempDCB$ = TempDCB$ & CRLF & "EOF character decimal =" & Str$(Asc(PassedDCB.EofChar))
  TempDCB$ = TempDCB$ & CRLF & "Event character decimal =" & Str$(Asc(PassedDCB.EvtChar))
  TempDCB$ = TempDCB$ & CRLF & "Transmit Delay =" & Str$(PassedDCB.TxDelay)
  DisplayDCB = TempDCB$
End Function

Function DisplayOpenCommError (PassedError As Integer) As String
'Used to display the error from the API "OpenComm"

Select Case PassedError
    Case IE_BADID
      DisplayOpenCommError = "Invalid. "
    Case IE_BAUDRATE
      DisplayOpenCommError = "Invalid baud rate. "
    Case IE_BYTESIZE
      DisplayOpenCommError = "Invalid byte size. "
    Case IE_DEFAULT
      DisplayOpenCommError = "Invalid default parameters. "
    Case IE_HARDWARE
      DisplayOpenCommError = "Not available. "
    Case IE_MEMORY
      DisplayOpenCommError = "Insufficient memory for I/O buffers. "
    Case IE_NOPEN
      DisplayOpenCommError = "Cannot be opened. "
    Case IE_OPEN
      DisplayOpenCommError = "In use. "
    Case Is > 0
      DisplayOpenCommError = "Available. "
    Case Else
      DisplayOpenCommError = "Unanticipated error#" & Str$(PassedError) & ". "
    End Select
End Function

Sub HardCodeDCB ()
'Uses global variables Temp_DCB and Temp_nCid to build a default DCB setup
'I use this instead of the API "BuildCommDCB"

  Temp_DCB.Id = Chr$(Temp_nCid)       '  identifies the comm port
  Temp_DCB.BaudRate = 1200            '  data transfer rate
  Temp_DCB.ByteSize = Chr$(8)         '  bits per data character
  Temp_DCB.Parity = Chr$(NOPARITY)    '  constant specifies type of parity
  Temp_DCB.StopBits = Chr$(ONESTOPBIT)'  constant specifies number of stop bits
  Temp_DCB.RlsTimeout = IGNORE        '  milliseconds to wait for CD
  Temp_DCB.CtsTimeout = IGNORE        '  milliseconds to wait for CTS input
  Temp_DCB.DsrTimeout = IGNORE        '  milliseconds to wait for DSR input
  
  'remark out all "Bits" lines you don't want
  Temp_DCB.Bits1 = Chr$(0)
  Temp_DCB.Bits1 = Chr$(Asc(Temp_DCB.Bits1) + DCB1_BINARY)      '  Binary mode ignores EOF character
  'Temp_DCB.Bits1 = Chr$(Asc(Temp_DCB.Bits1) + DCB1_RTSDISABLE)  '  RTS disable (Don't send RTS when port opened)
  'Temp_DCB.Bits1 = Chr$(Asc(Temp_DCB.Bits1) + DCB1_PARITY)      '  Turn on parity checking
  'Temp_DCB.Bits1 = Chr$(Asc(Temp_DCB.Bits1) + DCB1_OUTXCTSFLOW) '  Use CTS for flow control (Stop tx if CTS not present)
  'Temp_DCB.Bits1 = Chr$(Asc(Temp_DCB.Bits1) + DCB1_OUTXDSRFLOW) '  Use DSR for flow control (Stop tx if DSR not present)
  'Temp_DCB.Bits1 = Chr$(Asc(Temp_DCB.Bits1) + DCB1_DTRDISABLE)  '  DTR disable (Don't send DTR when port opened)
  Temp_DCB.Bits2 = Chr$(0)
  'Temp_DCB.Bits2 = Chr$(Asc(Temp_DCB.Bits2) + DCB2_OUTX)    '  Use XON/XOFF for flow control (Stop/start tx on XOFF/XON reception)
  'Temp_DCB.Bits2 = Chr$(Asc(Temp_DCB.Bits2) + DCB2_INX)     '  Other guy monitors XON/XOFF. Send them when rx buffer fills/empties
  'Temp_DCB.Bits2 = Chr$(Asc(Temp_DCB.Bits2) + DCB2_PECHAR)  '  If DCB1_PARITY, substitute DCB.PeChar  for errored characters
  'Temp_DCB.Bits2 = Chr$(Asc(Temp_DCB.Bits2) + DCB2_NULL)    '  Discard null characters (ascii 0)
  'Temp_DCB.Bits2 = Chr$(Asc(Temp_DCB.Bits2) + DCB2_CHEVT)   '  Trigger immediate event if receive DCB.EvtChar
  'Temp_DCB.Bits2 = Chr$(Asc(Temp_DCB.Bits2) + DCB2_DTRFLOW) '  Use DTR for flow control. Toggle when rx buffer fills/empties
  'Temp_DCB.Bits2 = Chr$(Asc(Temp_DCB.Bits2) + DCB2_RTSFLOW) '  Use RTS for flow control. Toggle when rx buffer fills/empties
  
  Temp_DCB.XonChar = Chr$(17)          '  XON character usually ^Q CHR$(17)
  Temp_DCB.XoffChar = Chr$(19)         '  XOFF character usually ^S CHR$(19)
  Temp_DCB.XonLim = 256               '  send XON/DTR=1/RTS=1 when chars in rx buffer fall below this value
  Temp_DCB.XoffLim = 256              '  send XOFF/DTR=0/RTS=0 when chars in rx buffer exceed this value
  Temp_DCB.PeChar = Chr$(255)           '  character to sub for errored characters (ignore if fParity=0 or fPeChar=0)
  Temp_DCB.EofChar = Chr$(26)          '  EOF character usually ^Z CHR$(26) (ignored if fBinary=1)
  Temp_DCB.EvtChar = Chr$(3)          '  immediate event character usually ^C CHR$(3) (ignore if fChEvt=0)
  Temp_DCB.TxDelay = IGNORE           '  not implemented

End Sub

Function IsValidChar (PassedNumber As Variant) As Integer
'Makes sure nothing will crash if we try a CHR$() function

  If PassedNumber > (-1) And PassedNumber < 256 Then
     If CInt(PassedNumber) = PassedNumber Then
	IsValidChar = True
     Else
	IsValidChar = False
     End If
  Else
     IsValidChar = False
  End If

End Function

Function IsValidNum (PassedNumber As Variant) As Integer
'Checks for non-negative integer values

  If PassedNumber > (-1) And PassedNumber < 32768 Then
     If CInt(PassedNumber) = PassedNumber Then
	IsValidNum = True
     Else
	IsValidNum = False
     End If
  Else
     IsValidNum = False
  End If

End Function

Sub PicturePrint (Destination As PictureBox, Buffer As String)
'Prints to a "PictureBox" and scrolls it like a terminal

Dim Yamount As Integer, Cnt As Integer
    Destination.ScaleMode = 3                ' Set to pixels
    Yamount = Destination.TextHeight("a")
    For Cnt = 1 To Len(Buffer)   ' Process all characters
	C$ = Mid$(Buffer, Cnt, 1)
	If C$ = Chr$(10) And Destination.CurrentY >= Destination.ScaleHeight - 2 * Yamount Then ' do we scroll?
	    ScrollWindow Destination.hWnd, 0, -Yamount, ByVal 0&, ByVal 0&
	    UpdateWindow Destination.hWnd
	    DoEvents
	    Destination.CurrentX = 0
	ElseIf C$ = Chr$(13) Then       ' Carriage returns reset line position
	    Destination.CurrentX = 0
	Else
	    Destination.Print C$;
	End If
    Next Cnt
End Sub

Sub TextPrint (DisplayBox As TextBox, Buffer As String)
'prints to a text box and scrolls it like a terminal

  DisplayBox.SelStart = Len(DisplayBox.Text) + 1
  DisplayBox.SelLength = 0
  DisplayBox.SelText = Buffer

End Sub

